# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 14:15:32 2025

@author: mstep
"""

#%%
import tkinter as tk
from PIL import Image, ImageDraw, ImageTk, ImageFont
from tkinter import ttk

class PilBboxTextApp:
    def __init__(self):
        self.window = tk.Tk()

        self.window.title('pil_text_bbox')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('300x300')

        # Drawing parameters.
        wid = 510
        hgt = 300
        margin = 10

        # Make a frame holding radio buttons.
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, anchor=tk.W, padx=5, pady=(5,0))

        # Font.
        label = tk.Label(frame, text='Font:')
        label.pack(side=tk.LEFT)
        self.font_var = tk.StringVar(value='times.ttf')
        combo = ttk.Combobox(frame, textvariable=self.font_var,
            values=['times.ttf', 'VIVALDII.TTF', 'MISTRAL.TTF', 'OLDENGL.TTF'])
        combo.pack(side=tk.LEFT)

        # Alignment.
        label = tk.Label(frame, text='Alignment:')
        label.pack(side=tk.LEFT, padx=(20,0))
        self.alignment_value = tk.StringVar(value='left')
        left_button = tk.Radiobutton(frame, variable=self.alignment_value,
            command=self.draw_samples, text='Left', value='left')
        left_button.pack(side=tk.LEFT, padx=10)
        center_button = tk.Radiobutton(frame, variable=self.alignment_value,
            command=self.draw_samples, text='Center', value='center')
        center_button.pack(side=tk.LEFT, padx=10)
        right_button = tk.Radiobutton(frame, variable=self.alignment_value,
            command=self.draw_samples, text='Right', value='right')
        right_button.pack(side=tk.LEFT, padx=10)

        # Make the canvas.
        self.canvas = tk.Canvas(self.window, bg='white',
            width=wid, height=hgt,
            borderwidth=0, highlightthickness=0)
        self.canvas.pack(side=tk.TOP, anchor=tk.NW, padx=margin, pady=margin)

        # Display the initial samples.
        self.draw_samples()

        # Do this after everything is ready.
        combo.bind("<<ComboboxSelected>>", lambda event: self.draw_samples())

        # Size the window to fit.
        self.window.geometry(f'{wid + 2 * margin}x{hgt + 2 * margin}')

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def draw_samples(self):
        '''Draw text samples.'''
        # Create the PIL image.
        wid = 510
        hgt = 300
        image = Image.new('RGB', (wid, hgt), 'white')

        # Create a Draw object to draw on the image.
        dr = ImageDraw.Draw(image)

        # Get the font.
        font_size = 20
        font = ImageFont.truetype(self.font_var.get(), font_size)

        # Get the selected alignment.
        align = self.alignment_value.get()

        # Draw the samples.
        # Column 1.
        x = 15
        y = 15
        self.draw_sample(dr, font, x, y, align, 'la')

        y += 6 * font_size
        self.draw_sample(dr, font, x, y, align, 'lm')

        y += 6 * font_size
        self.draw_sample(dr, font, x, y, align, 'ld')

        # Column 2.
        x += 12 * font_size
        y = 15
        self.draw_sample(dr, font, x, y, align, 'ma')

        y += 6 * font_size
        self.draw_sample(dr, font, x, y, align, 'mm')

        y += 6 * font_size
        self.draw_sample(dr, font, x, y, align, 'md')

        # Column 3.
        x += 12 * font_size
        y = 15
        self.draw_sample(dr, font, x, y, align, 'ra')

        y += 6 * font_size
        self.draw_sample(dr, font, x, y, align, 'rm')

        y += 6 * font_size
        self.draw_sample(dr, font, x, y, align, 'rd')


        # Convert the image into a PhotoImage for tkinter to display.
        self.photo_image = ImageTk.PhotoImage(image)

        # Display the result.
        self.canvas.delete(tk.ALL)
        self.canvas.create_image(0, 0, image=self.photo_image, anchor=tk.NW)

    def draw_sample(self, dr, font, x, y, align, anchor):
        # Draw a sample.
        # For quick informtion about font metrics, see:
        #   https://stackoverflow.com/questions/43060479/how-to-get-the-font-pixel-height-using-pils-imagefont-class
        radius = 3
        text = f'A longish first line\nAnchor: {anchor}\nEnding line'
        dr.multiline_text((x, y), text=text,
            fill='blue', font=font, anchor=anchor, align=align)
        dr.ellipse((x - radius, y - radius, x + radius, y + radius),
            fill='yellow', outline='black')

        # Get the bounding box.
        bbox = dr.textbbox((x, y), text, font=font)

        # Get the vertical offset from the anchor point
        # to the top of the first letters.
        (width, baseline), (offset_x, offset_y) = font.font.getsize(text)

        # Move the bounding box's up by offset_y and left by offset_x.
        bbox = [
            bbox[0] - offset_x, bbox[1] - offset_y,
            bbox[2], bbox[3]]

        # Adjust the bounding box depending on the anchor value.
        wid = bbox[2] - bbox[0]
        hgt = bbox[3] - bbox[1]
        print(f'point=({x}, {y}), {bbox=}, {offset_x=}, {offset_y=}, {wid=}, {hgt=}')

        if anchor[0] == 'm':     # Anchor point is in the middle horizontally
            bbox[0] -= wid // 2
            bbox[2] -= wid // 2
        elif anchor[0] == 'r':   # Anchor point is on the right horizontally
            bbox[0] -= wid
            bbox[2] -= wid
        if anchor[1] == 'm':     # Anchor point is in the middle vertically
            bbox[1] -= hgt // 2
            bbox[3] -= hgt // 2
        elif anchor[1] == 'd':   # Anchor point is on the bottom vertically
            bbox[1] -= hgt
            bbox[3] -= hgt

        dr.rectangle(bbox, outline='red')

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

#%%
PilBboxTextApp()
print('Done')
